package cn.bunny.controller;

import cn.bunny.common.spzx.model.dto.system.SysRoleDto;
import cn.bunny.common.spzx.model.entity.system.SysRole;
import cn.bunny.common.spzx.model.vo.common.Result;
import cn.bunny.common.spzx.model.vo.common.ResultCodeEnum;
import cn.bunny.service.SysRoleService;
import com.github.pagehelper.PageInfo;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import lombok.extern.log4j.Log4j2;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.Map;

@Tag(name = "用户角色查询")
@Log4j2
@RestController
@RequestMapping("/admin/system/sysRole")
public class SysRoleController {
    @Autowired
    private SysRoleService sysRoleService;

    @Operation(summary = "查询所有角色", description = "查询所有角色信息")
    @GetMapping("/findAllRoles/{userId}")
    public Result findAllRoles(@PathVariable("userId") Long userId) {
        Map<String, Object> map = sysRoleService.findAll(userId);
        return Result.build(map, ResultCodeEnum.SUCCESS);
    }

    @Operation(summary = "彻底删除角色", description = "使用id彻底删除角色信息")
    @DeleteMapping("deleteSysRoleByRoleId")
    public Result deleteSysRoleByRoleId(@RequestParam("roleId") Long roleId) {
        log.info("delete --- 彻底删除角色");

        sysRoleService.deleteSysRoleByRoleId(roleId);
        return Result.build(null, ResultCodeEnum.SUCCESS);
    }

    @Operation(summary = "删除角色", description = "使用id删除角色，将状态修改为1")
    @PutMapping("deleteSysRole")
    public Result deleteById(@RequestParam("roleId") Long roleId) {
        log.info("update --- 删除角色");

        sysRoleService.deleteSysRole(roleId);
        return Result.build(roleId, ResultCodeEnum.SUCCESS);
    }

    @Operation(summary = "角色修改方法", description = "修改角色")
    @PutMapping("updateSysRole")
    public Result updateSysRole(@RequestBody SysRole sysRole) {
        log.info("put --- 角色修改方法");

        sysRoleService.updateSysRole(sysRole);
        return Result.build(sysRole, ResultCodeEnum.SUCCESS);
    }

    @Operation(summary = "角色添加的方法", description = "角色添加")
    @PostMapping(value = "saveSysRole")
    public Result saveSysRole(@RequestBody SysRole sysRole) {
        log.info("post --- 角色添加的方法");

        sysRoleService.saveSysRole(sysRole);
        return Result.build(sysRole, ResultCodeEnum.SUCCESS);
    }

    @Operation(summary = "角色列表查询", description = "角色列表查询")
    @PostMapping("findByPage/{current}/{limit}")
    public Result findByPage(@PathVariable("current") Integer current,
                             @PathVariable("limit") Integer limit,
                             @RequestBody SysRoleDto sysRoleDto) {
        log.info("post --- 角色列表查询");

        PageInfo<SysRole> pageInfo = sysRoleService.findByPage(sysRoleDto, current, limit);
        return Result.build(pageInfo, ResultCodeEnum.SUCCESS);
    }
}
