package cn.bunny.service.impl;

import cn.bunny.common.exception.BunnyException;
import cn.bunny.common.spzx.model.entity.product.Category;
import cn.bunny.common.spzx.model.vo.common.ResultCodeEnum;
import cn.bunny.common.spzx.model.vo.product.CategoryExcelVo;
import cn.bunny.lister.ExcelLister;
import cn.bunny.mapper.CategoryMapper;
import cn.bunny.service.CategoryService;
import com.alibaba.excel.EasyExcel;
import jakarta.servlet.http.HttpServletResponse;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;
import org.springframework.web.multipart.MultipartFile;

import java.io.IOException;
import java.net.URLEncoder;
import java.nio.charset.StandardCharsets;
import java.util.ArrayList;
import java.util.List;

@Service
public class CategoryServiceImpl implements CategoryService {
    @Autowired
    private CategoryMapper categoryMapper;

    // 根据id查询分类列表
    @Override
    public List<Category> findCategoryList(Long id) {
        List<Category> categoryList = categoryMapper.selectCategoryByParentId(id);

        if (!CollectionUtils.isEmpty(categoryList)) {
            categoryList.forEach(category -> {
                int count = categoryMapper.selectCountByParentId(category.getId());

                category.setHasChildren(count > 0);
            });
        }
        return categoryList;
    }

    // 导出Excel
    @Override
    public void exportData(HttpServletResponse response) {
        try {
            // 1. 设置响应头信息
            response.setContentType("application/vnd.ms-excel");
            response.setCharacterEncoding("utf-8");

            String filename = URLEncoder.encode("分类数据", StandardCharsets.UTF_8);

            response.setHeader("Content-dispostion", "attachment;filename=" + filename + ".xlsx");
            // 2. 调用mapper，查询所有分类，返回list集合
            List<Category> categoryList = categoryMapper.findAll();
            List<CategoryExcelVo> categoryExcelVoArrayList = new ArrayList<>();

            for (Category category : categoryList) {
                CategoryExcelVo categoryExcelVo = new CategoryExcelVo();
                BeanUtils.copyProperties(category, categoryExcelVo);
                categoryExcelVoArrayList.add(categoryExcelVo);
            }

            EasyExcel.write(response.getOutputStream(), CategoryExcelVo.class)
                    .sheet("分类数据")
                    .doWrite(categoryExcelVoArrayList);
        } catch (Exception e) {
            e.printStackTrace();
            throw new BunnyException(ResultCodeEnum.DATA_ERROR);
        }

    }

    // 导入文件
    @Override
    public void importData(MultipartFile file) {
        try {
            // 创建监听器对象，传递mapper对象
            ExcelLister<CategoryExcelVo> excelListener = new ExcelLister<>(categoryMapper);
            // 调用read方法读取excel数据
            EasyExcel.read(file.getInputStream(),
                    CategoryExcelVo.class,
                    excelListener).sheet().doRead();
        } catch (IOException e) {
            throw new BunnyException(ResultCodeEnum.DATA_ERROR);
        }
    }
}