package cn.bunny.service.impl;

import cn.bunny.common.exception.BunnyException;
import cn.bunny.common.spzx.model.vo.common.ResultCodeEnum;
import cn.bunny.properties.MinioProperties;
import cn.bunny.service.FIleUploadService;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.lang.UUID;
import io.minio.BucketExistsArgs;
import io.minio.MakeBucketArgs;
import io.minio.MinioClient;
import io.minio.PutObjectArgs;
import lombok.extern.log4j.Log4j2;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;

import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.Date;

@Log4j2
@Service
public class FileUploadServiceImpl implements FIleUploadService {
    @Autowired
    private MinioProperties minioProperties;

    @Override
    public String uploadFile(MultipartFile multipartFile) {

        try {
            // 创建一个Minio的客户端对象
            MinioClient minioClient = MinioClient.builder()
                    .endpoint(minioProperties.getEndpointUrl())
                    .credentials(minioProperties.getAccessKey(), minioProperties.getSecretKey())
                    .build();

            // 判断桶是否存在
            boolean found = minioClient.bucketExists(BucketExistsArgs
                    .builder()
                    .bucket(minioProperties.getBucketName())
                    .build());

            if (!found) {       // 如果不存在，那么此时就创建一个新的桶
                minioClient.makeBucket(MakeBucketArgs
                        .builder()
                        .bucket(minioProperties.getBucketName())
                        .build());
            }

            // 设置存储对象名称
            String dateDir = DateUtil.format(new Date(), "yyyy-MM-dd");
            String uuid = UUID.randomUUID().toString().replace("-", "");
            String fileName = dateDir + "/" + uuid + multipartFile.getOriginalFilename();

            // 存储文件
            PutObjectArgs putObjectArgs = PutObjectArgs.builder()
                    .bucket(minioProperties.getBucketName())
                    .stream(multipartFile.getInputStream(), multipartFile.getSize(), -1)
                    .object(fileName)
                    .build();
            minioClient.putObject(putObjectArgs);

            // 返回访问地址
            return minioProperties.getEndpointUrl() + "/" + minioProperties.getBucketName() + "/" + fileName;
        } catch (Exception e) {
            e.printStackTrace();
            throw new BunnyException(ResultCodeEnum.SYSTEM_ERROR);
        }
    }


    // 保存文件
    @Override
    public void saveFile(MultipartFile multipartFile) throws IOException {
        // 生成唯一的文件名
        String fileName = multipartFile.getOriginalFilename();
        // 创建上传目录（如果不存在）
        Path uploadDir = Paths.get("F:\\File\\");
        Files.createDirectories(uploadDir);
        // 保存文件到本地文件系统
        Path filePath = uploadDir.resolve(fileName);
        Files.write(filePath, multipartFile.getBytes());
    }
}
